import { DataSource, ViewColumn, ViewEntity } from "typeorm";
import { membership } from "../entity/club/member/membership";
import { Salutation } from "../enums/salutation";

@ViewEntity({
  expression: (datasource: DataSource) =>
    datasource
      .getRepository(membership)
      .createQueryBuilder("membership")
      .select("status.id", "statusId")
      .addSelect("status.status", "status")
      .addSelect("member.id", "memberId")
      .addSelect("member.salutation", "memberSalutation")
      .addSelect("member.firstname", "memberFirstname")
      .addSelect("member.lastname", "memberLastname")
      .addSelect("member.nameaffix", "memberNameaffix")
      .addSelect("member.birthdate", "memberBirthdate")
      .addSelect("SUM(TIMESTAMPDIFF(DAY, membership.start, COALESCE(membership.end, CURRENT_DATE)))", "durationInDays")
      .addSelect(
        "CONCAT('_', FROM_DAYS(SUM(TIMESTAMPDIFF(DAY, membership.start, COALESCE(membership.end, CURRENT_DATE)))))",
        "durationInYears"
      )
      .leftJoin("membership.status", "status")
      .leftJoin("membership.member", "member")
      .groupBy("status.id")
      .addGroupBy("member.id"),
})
export class membershipView {
  @ViewColumn()
  durationInDays: number;

  @ViewColumn()
  durationInYears: Date;

  @ViewColumn()
  status: string;

  @ViewColumn()
  statusId: number;

  @ViewColumn()
  memberId: number;

  @ViewColumn()
  memberSalutation: Salutation;

  @ViewColumn()
  memberFirstname: string;

  @ViewColumn()
  memberLastname: string;

  @ViewColumn()
  memberNameaffix: string;

  @ViewColumn()
  memberBirthdate: Date;
}
